//
//  VoltageOutputController.m
//  VoltageOutput
//
//  Created by Phidgets on 2015-07-23.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "VoltageOutputController.h"

@implementation VoltageOutputController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close the example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetVoltageOutput_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetVoltageOutput_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks]
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, and error events
- (void)onAttachHandler{
    double minVoltage,maxVoltage, voltage;
    Phidget_DeviceID deviceID;
    int enabled;
    PhidgetVoltageOutput_VoltageOutputRange voltageOutputRange;
    
    //Get information from channel that will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetVoltageOutput_getMinVoltage(ch, &minVoltage);
    PhidgetVoltageOutput_getMaxVoltage(ch, &maxVoltage);
    PhidgetVoltageOutput_getVoltage(ch, &voltage);
    PhidgetVoltageOutput_getEnabled(ch, &enabled);
    PhidgetVoltageOutput_getVoltageOutputRange(ch, &voltageOutputRange);
    
    //Adjust GUI based on information from the channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [outputBox setHidden:NO];
    
    switch(deviceID){
        case PHIDID_1002:
        case PHIDID_OUT1000:
            [outputModeBox setHidden:YES];
            [outputEnableBox setHidden:NO];
            [enableButton setIntValue:enabled];
            if(enabled){
                [enableButton setTitle:@"Disable"];
            }
            else{
                [enableButton setTitle:@"Enable"];
            }
            break;
        case PHIDID_OUT1001:
        case PHIDID_OUT1002:
            [outputModeBox setHidden:NO];
            break;
        default:
            break;
    }
    
    //Configure voltage output slider with information read from channel
    [voltageOutputSlider setMaxValue:maxVoltage];
    [voltageOutputSlider setMinValue:minVoltage];
    [voltageOutputSlider setDoubleValue:voltage];
    [voltageOutputMinLabel setStringValue:[NSString stringWithFormat:@"%.2f",minVoltage]];
    [voltageOutputMaxLabel setStringValue:[NSString stringWithFormat:@"%.2f",maxVoltage]];
    [voltageOutputText setStringValue:[NSString stringWithFormat:@"%.2f V",voltage]];
    
    //Configure voltage output button based on information read from channel
    if(voltageOutputRange == VOLTAGE_OUTPUT_RANGE_5V){
        [voltageModeButton setTitle:@"Switch to 10V"];
    }
    else if(voltageOutputRange == VOLTAGE_OUTPUT_RANGE_10V){
        [voltageModeButton setTitle:@"Switch to 5V"];
    }

    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}
- (void)onDetachHandler{
    //Reset
    [outputBox setHidden:YES];
    [outputModeBox setHidden:YES];
    [outputEnableBox setHidden:YES];
    [voltageOutputText setStringValue:@""];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

#pragma mark GUI controls
- (IBAction)setEnabled:(id)sender{
    PhidgetReturnCode result;
    int enabled;
    
    result = PhidgetVoltageOutput_getEnabled(ch, &enabled);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    
    if(enabled){
        result = PhidgetVoltageOutput_setEnabled(ch, 0);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
        [enableButton setTitle:@"Enable"];
    }
    else{
        result = PhidgetVoltageOutput_setEnabled(ch, 1);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
        [enableButton setTitle:@"Disable"];
    }
}

- (IBAction)setVoltage:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetVoltageOutput_setVoltage(ch, [voltageOutputSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [voltageOutputText setStringValue:[NSString stringWithFormat:@"%0.2f V", [voltageOutputSlider doubleValue]]];
}
- (IBAction)setVoltageOutputMode:(id)sender{
    double minVoltage, maxVoltage, voltage;
    PhidgetReturnCode result;
    if([[voltageModeButton title]  isEqual: @"Switch to 10V"]){
        result = PhidgetVoltageOutput_setVoltageOutputRange(ch, VOLTAGE_OUTPUT_RANGE_10V);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
        [voltageModeButton setTitle:@"Switch to 5V"];
    }
    else{
        result = PhidgetVoltageOutput_setVoltageOutputRange(ch, VOLTAGE_OUTPUT_RANGE_5V);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
        [voltageModeButton setTitle:@"Switch to 10V"];
    }
    PhidgetVoltageOutput_getMinVoltage(ch, &minVoltage);
    PhidgetVoltageOutput_getMaxVoltage(ch, &maxVoltage);
    PhidgetVoltageOutput_getVoltage(ch, &voltage);
    [voltageOutputSlider setMinValue:minVoltage];
    [voltageOutputSlider setMaxValue:maxVoltage];
    [voltageOutputSlider setDoubleValue:voltage];
    [voltageOutputMinLabel setStringValue:[NSString stringWithFormat:@"%.2f",minVoltage]];
    [voltageOutputMaxLabel setStringValue:[NSString stringWithFormat:@"%.2f",maxVoltage]];
    [voltageOutputText setStringValue:[NSString stringWithFormat:@"%.2f V",voltage]];
}

@end
